@@ -11,7 +11,7 @@ from ruamel.yaml import YAML
 
 from teko.helpers.clog import CLog
 from teko.helpers.exceptions import TekoJiraException
-from teko.models.jira_testcase import JiraTestcase, JiraTestScript, JiraTestScriptStep
+from teko.models.jira_testcase import JiraTestcase
 
 
 class JiraService:
@@ -40,11 +40,11 @@ class JiraService:
         jira_username = config.get('jira_username') or os.getenv('JIRA_USERNAME')
         jira_password = config.get('jira_password') or os.getenv('JIRA_PASSWORD')
         confluence_username = config.get('confluence_username') or os.getenv('CONFLUENCE_USERNAME')
-        confluence_password = config.get('confluence_password') or os.getenv('CONFLUENCE_PASSWOPRD')
+        confluence_password = config.get('confluence_password') or os.getenv('CONFLUENCE_PASSWORD')
 
-        if not self.project_key or not server or not jira_username or not jira_password:
-            msg = "[TEKO TOOL] No valid JIRA configuration found, please set JIRA_SERVER, JIRA_PROJECT_KEY, " \
-                  "JIRA_USERNAME and JIRA_PASSWORD environment variables first"
+        if not self.project_key or not server or not jira_username or not jira_password or not confluence_username or not confluence_password:
+            msg = "No valid JIRA configuration found, please set JIRA_SERVER, JIRA_PROJECT_KEY, JIRA_USERNAME, " \
+                  "JIRA_PASSWORD, CONFLUENCE_USERNAME and CONFLUENCE_PASSWORD environment variables first."
             CLog.error(msg)
             raise TekoJiraException(msg)
 
@@ -80,18 +80,18 @@ class JiraService:
         res = self.jira_session.get(url=url, params=params)
 
         if res.status_code != 200:
-            CLog.error(f"[TEKO TOOL] Failed to search for testcase {name}")
-            if res.status_code == 403:
-                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {url} with params {params}")
+            CLog.error(f"Failed to search for test case {name}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request GET to {url} with params {params}.")
             else:
-                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             return []
         raw = res.text
 
         testcases_json = json.loads(raw)
 
-        print(f"[TEKO TOOL] {json.dumps(testcases_json)}")
-
         testcases = []
         for t in testcases_json:
             testcases.append(JiraTestcase().from_dict(t))
@@ -103,17 +103,19 @@ class JiraService:
         res = self.jira_session.delete(url=url)
 
         if res.status_code != 200:
-            CLog.error(f"[TEKO TOOL] Failed to delete trace link {link_id}")
-            if res.status_code == 403:
-                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request DELETE to {url}")
+            CLog.error(f"Failed to delete trace link {link_id}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request DELETE to {url}.")
             else:
-                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             return
         raw = res.text
 
         testcases_json = json.loads(raw)
 
-        print(f"[TEKO TOOL] {json.dumps(testcases_json)}")
+        CLog.info(f"{json.dumps(testcases_json)}")
 
     def empty_trace_links(self, test_id):
         url_confluence = self.base_api_tests_url + f"/testcase/{test_id}/tracelinks/confluencepage"
@@ -121,11 +123,13 @@ class JiraService:
         res = self.jira_session.get(url=url_confluence)
 
         if res.status_code != 200:
-            CLog.error(f"[TEKO TOOL] Failed to clear confluence links for testcase {test_id}")
-            if res.status_code == 403:
-                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {url_confluence}")
+            CLog.error(f"Failed to clear confluence links for test case {test_id}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request GET to {url_confluence}.")
             else:
-                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
         else:
             raw = res.text
             links = json.loads(raw)
@@ -137,11 +141,13 @@ class JiraService:
         res = self.jira_session.get(url=url_web)
 
         if res.status_code != 200:
-            CLog.error(f"[TEKO TOOL] Failed to clear web links for testcase {test_id}")
-            if res.status_code == 403:
-                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {url_web}")
+            CLog.error(f"Failed to clear web links for test case {test_id}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request GET to {url_web}.")
             else:
-                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
         else:
             raw = res.text
             links = json.loads(raw)
@@ -159,16 +165,18 @@ class JiraService:
         for confluence_url in confluences:
             res = self.confluence_session.get(url=confluence_url)
             if res.status_code != 200:
-                CLog.error(f"[TEKO TOOL] Failed to get id of confluence page {confluence_url}")
-                if res.status_code == 403:
-                    CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {confluence_url}")
+                CLog.error(f"Failed to get id of confluence page {confluence_url}.")
+                if res.status_code == 401:
+                    CLog.error(f"Unauthorized: Authenticated failed.")
+                elif res.status_code == 403:
+                    CLog.error(f"Forbidden: You don't have permission to send request GET to {confluence_url}.")
                 else:
-                    CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+                    CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
                 self.success = False
                 return
 
             html = res.text
-            soup = BeautifulSoup(html.text, "html.parser")
+            soup = BeautifulSoup(html, "html.parser")
             page_id = soup.find("meta", {"name": "ajs-page-id"})["content"]
 
             confluences_data.append({
@@ -180,11 +188,13 @@ class JiraService:
         res = self.jira_session.post(url=url, json=confluences_data)
 
         if res.status_code != 200:
-            CLog.error(f"[TEKO TOOL] Failed to add confluence links for testcase {test_id}")
-            if res.status_code == 403:
-                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {confluences_data}")
+            CLog.error(f"Failed to add confluence links for test case {test_id}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {confluences_data}.")
             else:
-                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             self.success = False
 
     def add_web_links(self, test_id, webs: List[Union[str, dict]]):
@@ -214,11 +224,13 @@ class JiraService:
         res = self.jira_session.post(url=url, json=webs_data)
 
         if res.status_code != 200:
-            CLog.error(f"[TEKO TOOL] Failed to add web links for testcase {test_id}")
-            if res.status_code == 403:
-                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {webs_data}")
+            CLog.error(f"Failed to add web links for test case {test_id}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {webs_data}.")
             else:
-                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             self.success = False
 
     def add_links(self, test_id, confluences: List[str], webs: List[dict]):
@@ -241,22 +253,27 @@ class JiraService:
         Ignore failure
         """
         res = self.jira_session.post(url=url, json=data)
+        if res.status_code == 401:
+            CLog.error(f"Cannot create {type} folder {name}.")
+            CLog.error(f"Unauthorized: Authenticated failed.")
         if res.status_code == 403:
-            CLog.error(f"[TEKO TOOL] Cannot create {type} folder {name}")
-            CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {data}")
+            CLog.error(f"Cannot create {type} folder {name}.")
+            CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {data}.")
         if res.status_code != 201:
-            CLog.info(f"[TEKO TOOL] {type} Folder {name} has already exists")
+            CLog.info(f"{type} Folder {name} has already exists.")
 
     def get_test_id(self, test_key):
         url = self.base_api_tests_url + f"/testcase/{test_key}"
 
         res = self.jira_session.get(url=url, params={"fields": "id"})
         if res.status_code != 200:
-            CLog.error(f"[TEKO TOOL] Failed to get id of testcase {test_key}")
-            if res.status_code == 403:
-                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {url}")
+            CLog.error(f"Failed to get id of test case {test_key}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request GET to {url}.")
             else:
-                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}")
             self.success = False
             return
 
@@ -264,16 +281,25 @@ class JiraService:
         return json.loads(raw).get('id')
 
     def create_or_update_testcases_by_name(self, testcases: List[JiraTestcase], issues=[]):
+        CLog.info("Creating/Updating test cases.")
         self.success = True
+
+        folders = set()
+
+        for testcase in testcases:
+            if testcase.folder:
+                folders.add(testcase.folder)
+
+        for folder in folders:
+            self.create_folder(folder, "TEST_CASE")
+
         for testcase in testcases:
             existing_testcases = self.search_testcases(issues=issues, name=testcase.name)
             if not existing_testcases:
                 key = self.create_testcase(testcase)
-                CLog.info(f"[TEKO TOOL] New testcase created: {key}")
                 testcase.key = key
             else:
                 key = self.update_testcase(existing_testcases[0].key, testcase)
-                CLog.info(f"[TEKO TOOL] Existing testcase updated: {key}")
                 testcase.key = key
             if key:
                 self.add_links(
@@ -282,8 +308,8 @@ class JiraService:
                     webs=testcase.webLinks
                 )
         if not self.success:
-            msg = "[TEKO TOOL] Creating/updating test cases process do not succeeded"
-            CLog.error(msg)
+            msg = "Creating/updating test cases process does not succeeded."
+            CLog.info(msg)
             raise TekoJiraException(msg)
         return testcases
 
@@ -313,8 +339,6 @@ class JiraService:
 
         if testcase.folder == "":
             data.pop("folder")
-        else:
-            self.create_folder(testcase.folder, "TEST_CASE")
 
         if testcase.issueLinks:
             data['issueLinks'] = testcase.issueLinks
@@ -328,14 +352,17 @@ class JiraService:
         res = self.jira_session.put(url=url, json=data)
 
         if res.status_code != 200:
-            CLog.error(f"[TEKO TOOL] Failed to update testcase `{testcase.name}`")
-            if res.status_code == 403:
-                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request PUT to {url} with payload {data}")
+            CLog.error(f"Failed to update test case `{testcase.name}`.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request PUT to {url} with payload {data}.")
             else:
-                CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             self.success = False
             return None
 
+        CLog.info(f"Updated test `{testcase.name}` successfully. Test key: {key}.")
         return key
 
     def create_testcase(self, testcase: JiraTestcase):
@@ -343,21 +370,23 @@ class JiraService:
         url = self.base_api_url + "/testcase"
         res = self.jira_session.post(url=url, json=data)
 
-        if res.status_code != 200:
-            CLog.error(f"[TEKO TOOL] Failed to create testcase `{testcase.name}`")
-            if res.status_code == 403:
-                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {data}")
+        if res.status_code != 201:
+            CLog.error(f"Failed to create test case `{testcase.name}`.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {data}.")
             else:
-                CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             self.success = False
             return None
 
         raw = res.text
 
         testcase_json = json.loads(raw)
-        print(f"[TEKO TOOL] {json.dumps(testcase_json)}")
-
-        return testcase_json['key']
+        key = testcase_json['key']
+        CLog.info(f"Created test `{testcase.name}` successfully. Test key: {key}.")
+        return key
 
     def prepare_testrun_data(self, testcase: JiraTestcase):
         if not testcase.projectKey:
@@ -374,21 +403,31 @@ class JiraService:
         return data
 
     def create_test_cycle(self, testcases: List[JiraTestcase]):
+        CLog.info("Creating test cycles.")
         self.success = True
         items_by_folder = dict()
         for testcase in testcases:
             existing_testcases = self.search_testcases(name=testcase.name)
             if not existing_testcases:
-                CLog.warn(f"[TEKO TOOL] Testcase with name `{testcase.name}` not existed!")
+                CLog.error(f"Testcase with name `{testcase.name}` not existed.")
+                self.success = False
             else:
                 testcase.key = existing_testcases[0].key
-                CLog.info(f"[TEKO TOOL] Testcase with name `{testcase.name}` existed: #{testcase.key}")
+                CLog.info(f"Testcase with name `{testcase.name}` existed.")
                 if items_by_folder.get(testcase.testrun_folder) is None:
                     items_by_folder[testcase.testrun_folder] = []
                 items_by_folder[testcase.testrun_folder].append(self.prepare_testrun_data(testcase))
 
+        if not self.success:
+            msg = "Creating test cycle process does not succeeded."
+            CLog.error(msg)
+            raise TekoJiraException(msg)
+
         url = self.base_api_url + "/testrun"
 
+        for folder in items_by_folder.keys():
+            self.create_folder(folder, "TEST_RUN")
+
         for folder, items in items_by_folder.items():
             data = {
                 "projectKey": self.project_key,
@@ -399,34 +438,34 @@ class JiraService:
 
             if not folder:
                 data.pop("folder")
-            else:
-                self.create_folder(folder, "TEST_RUN")
-
-            print(f"[TEKO TOOL] Data: {data}")
 
             res = self.jira_session.post(url=url, json=data)
 
-            if res.status_code != 200:
-                CLog.error(f"[TEKO TOOL] Failed to create test cycle`")
-                if res.status_code == 403:
-                    CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {data}")
+            if res.status_code != 201:
+                CLog.error(f"Failed to create test cycle.")
+                if res.status_code == 401:
+                    CLog.error(f"Unauthorized: Authenticated failed.")
+                elif res.status_code == 403:
+                    CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {data}.")
                 else:
-                    CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+                    CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
                 self.success = False
                 return
 
             raw = res.text
 
             testcase_json = json.loads(raw)
-            print(f"[TEKO TOOL] {json.dumps(testcase_json)}")
-
             key = testcase_json['key']
-            CLog.info(f"[TEKO TOOL] Test cycle `{key}` created!")
+            if folder:
+                CLog.info(f"Test cycle {key} created in folder {folder}.")
+            else:
+                CLog.info(f"Test cycle {key} created.")
+
         if not self.success:
-            msg = "[TEKO TOOL] Creating test cycle process does not succeeded"
+            msg = "Creating test cycle process does not succeeded."
             CLog.error(msg)
             raise TekoJiraException(msg)
-
+        
     @staticmethod
     def read_testcases_from_file(testcase_file):
         """
@@ -435,13 +474,13 @@ class JiraService:
         :return:
         """
         filename, ext = os.path.splitext(testcase_file)
-        print(f"[TEKO TOOL] {os.path.abspath(testcase_file)}")
+        CLog.info(f"Reading {os.path.abspath(testcase_file)}.")
         if ext == ".json":
             with codecs.open(testcase_file, "r", encoding="utf-8") as f:
                 try:
                     testcase_data = json.load(f)
                 except Exception:
-                    msg = "[TEKO TOOL] Invalid json format"
+                    msg = "Invalid json format."
                     CLog.error(msg)
                     raise TekoJiraException(msg) from None
 
@@ -451,11 +490,11 @@ class JiraService:
                 try:
                     testcase_data = yaml.load(f)
                 except Exception:
-                    msg = "[TEKO TOOL] Invalid yaml format"
+                    msg = "Invalid yaml format"
                     CLog.error(msg)
                     raise TekoJiraException(msg) from None
         else:
-            msg = f"[TEKO TOOL] File format not supported: {ext}"
+            msg = f"File format not supported: {ext}."
             CLog.error(msg)
             raise TekoJiraException(msg)
 
@@ -468,7 +507,7 @@ class JiraService:
             try:
                 testcases.append(JiraTestcase().from_dict(t))
             except Exception:
-                msg = f"[TEKO TOOL] Failed to parse test data from file {testcase_file}"
+                msg = f"Failed to parse test data from file {testcase_file}."
                 CLog.error(msg)
                 raise TekoJiraException(msg) from None
 
@@ -483,12 +522,12 @@ class JiraService:
         """
         testcases_json = []
         for t in testcases:
-            print(f"[TEKO TOOL] {t}")
+            CLog.info(f"{t}")
             data = t.to_dict()
             testcases_json.append(data)
 
         filename, ext = os.path.splitext(testcase_file)
-        print(filename, ext)
+        CLog.info(filename, ext)
         if ext == ".json":
             with codecs.open(testcase_file, "w", encoding="utf-8") as f:
                 json.dump(testcases_json, f, indent=2)
@@ -498,29 +537,19 @@ class JiraService:
                 yaml.indent(offset=2)
                 yaml.dump(testcases_json, f)
         else:
-            msg = f"[TEKO TOOL] File format not supported: {ext}"
+            msg = f"File format not supported: {ext}."
             CLog.error(msg)
             raise TekoJiraException(msg) from None
 
-        return None
-
 
 if __name__ == "__main__":
     jira_srv = JiraService()
 
     testcase_file = "../../../sample/testcases.json"
     testcases = JiraService.read_testcases_from_file(testcase_file)
-
-    for t in testcases:
-        print(f"[TEKO TOOL] {t}")
-
     jira_srv.create_or_update_testcases_by_name(testcases)
 
-    print("\nAfter:\n")
-    for t in testcases:
-        print(f"[TEKO TOOL] {t}")
-
-    testcycle_file = "../../../sample/testcases.json"
+    testcycle_file = "../../../sample/testcycles.json"
     testcases = JiraService.read_testcases_from_file(testcycle_file)
     jira_srv.create_test_cycle(testcases)
 
